# -*- coding: utf-8 -*-
'''
    The venvm is small webtool for manager virtualenv.
    Copyright (C) 2013  Rodrigo Pinheiro Matias <rodrigopmatias@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see {http://www.gnu.org/licenses/}.
'''
from flask import Flask, render_template, session
from venvm.helpers import _, get_languages_options, get_i18n_filepath
from datetime import timedelta
from venvm.db import get_connection
from urllib2 import urlopen

import sys
import os
import re
import subprocess as sp
import venvm
import json

app = Flask(__name__)
app.permanent_session_lifetime = timedelta(minutes=10)
app.secret_key = '123'

DEFINED_MENU = {
    'actions': [
        {
            'href': 'manager',
            'title': u'Gerenciar Pacotes'
        },
        {
            'href': 'information',
            'title': u'Informações'
        }
    ],
    'sites': [
        {
            'href': 'python',
            'title': u'Python'
        },
        {
            'href': 'pypy',
            'title': u'Pypy'
        },
        {
            'href': 'pypi',
            'title': u'Python Packages'
        }
    ]
}

app.jinja_env.filters['_'] = _


@app.route('/package/info/<name>')
def package_information(name):
    resource = 'https://pypi.python.org/pypi/%s/json' % name
    rst = {}

    f = urlopen(resource)
    rst = json.load(f)

    return render_template('package.html', **rst)


@app.route('/package/list')
def package_list():
    conn = get_connection()
    cursor = conn.cursor()

    cursor.execute("select id, name, version from package order by name asc")

    rst = {
        'packages': [
            {
                'pk': pk,
                'name': name,
                'version': version,
                'last_version': None
            }
            for pk, name, version in cursor.fetchall()
        ]
    }

    cursor.close()
    conn.close()

    return render_template('list-item.html', **rst)


@app.route('/sync')
def sync():
    rst = {
        'success': True
    }

    os.chdir(os.environ.get('VIRTUAL_ENV'))
    pid = sp.Popen("pip freeze", shell=True, stdout=sp.PIPE)
    cnt = 0

    dba = get_connection()
    cursor = dba.cursor()

    try:
        for info in pid.stdout.read().split('\n'):
            if info != '':
                name, version = info.split('==')
                cursor.execute('select count(*) from package where name=?', [name])
                count, = cursor.fetchone()

                if count > 0:
                    dba.execute('update package set version=? where name=?', [version, name])
                else:
                    dba.execute('insert into package(name, version) values(?, ?)', [name, version])
            else:
                print('skip out')

            cnt += 1
    except Exception as e:
        rst.update(message=e.message, success=False)
    else:
        rst.update(message=_(u'Atualizada informação de %d pacotes') % cnt)

    cursor.close()
    dba.commit()
    dba.close()

    return json.dumps(rst, indent=4)


@app.route('/')
def index():
    params = {}

    params.update(
        menu=DEFINED_MENU,
        langs=get_languages_options()
    )
    return render_template('index.html', **params)


@app.route('/language/<lang>')
def language(lang):
    rst = {'success': False}

    if os.path.exists(get_i18n_filepath(lang)) is True:
        rst.update(success=True)
        session.update(lang=lang)

    return json.dumps(rst)


@app.route('/manager')
def manager():
    return render_template('manager.html')


def get_directory_size(directory):
    pid = sp.Popen(['/usr/bin/du', '-sh', directory], stdout=sp.PIPE)
    regex = re.compile('^(?P<size>\d+(M|G|T|H)).*$')

    rst = regex.match(pid.stdout.read())
    if rst is not None:
        return rst.groupdict().get('size')
    else:
        return 'undefined'


def get_virtualenv_version():
    pid = sp.Popen('type -p virtualenv', shell=True, stdout=sp.PIPE)
    path = pid.stdout.read().replace('\n', '')

    if os.path.isfile(path) and os.access(path, os.X_OK):
        pid = sp.Popen([path, '--version'], stdout=sp.PIPE)
        return pid.stdout.read().replace('\n', '')
    else:
        return 'undefined'


@app.route('/information')
def information():
    params = {
        'py_version': '%s.%s.%s' % (
            sys.version_info.major,
            sys.version_info.minor,
            sys.version_info.micro
        ),
        'py_platform': sys.platform,
        'py_api_version': sys.api_version,
        'py_copyright': sys.copyright,
        'venv_version': get_virtualenv_version(),
        'venvm_version': venvm.__version__,
        'venvm_codename': venvm.__codename__,
        'size': get_directory_size(os.environ.get('VIRTUAL_ENV')),
        'path': os.environ.get('VIRTUAL_ENV')
    }

    try:
        import pip
    except Exception:
        pass
    else:
        params.update(
            pip={
                'version': pip.__version__
            }
        )

    try:
        import setuptools
    except Exception:
        pass
    else:
        params.update(
            setuptools={
                'version': setuptools.__version__
            }
        )

    try:
        import distutils
    except Exception:
        pass
    else:
        params.update(
            distutils={
                'version': distutils.__version__
            }
        )

    return render_template('information.html', **params)
